@@ -21,10 +21,11 @@ except ModuleNotFoundError:
     logger.warning("Support for AutoARIMA is not available. To enable it, install u8darts[pmdarima] or u8darts[all].")
 
 try:
-    from darts.models.forecasting.prophet import Prophet
+    from darts.models.forecasting.prophet_model import Prophet
 except ModuleNotFoundError:
-    logger.warning("Support Facebook Prophet is not available. "
-                   "To enable it, install u8darts[prophet] or u8darts[all].")
+    logger.warning('Support for Facebook Prophet is not available. '
+                   'To enable it, install "darts", "u8darts[prophet]" or "u8darts[all]" (with pip);'
+                   'or "u8darts-all" (with conda).')
 
 try:
     from darts.models.forecasting.block_rnn_model import BlockRNNModel
@@ -32,18 +33,28 @@ try:
     from darts.models.forecasting.tcn_model import TCNModel
     from darts.models.forecasting.nbeats import NBEATSModel
     from darts.models.forecasting.transformer_model import TransformerModel
+    from darts.models.forecasting.tft_model import TFTModel
 
 except ModuleNotFoundError:
-    logger.warning("Support Torch based models not available. To enable it, install u8darts[torch] or u8darts[all].")
+    logger.warning('Support for Torch based models not available. '
+                   'To enable them, install "darts", "u8darts[torch]" or "u8darts[all]" (with pip); '
+                   'or "u8darts-torch" or "u8darts-all" (with conda).')
 
-# Regression
-from darts.models.forecasting.lgbm_model import LinearRegressionModel
-from darts.models.forecasting.random_forest import RandomForest
-from darts.models.forecasting.regression_model import RegressionModel
+# Regression & RegressionEnsemble:
+try:
+    from darts.models.forecasting.linear_regression_model import LinearRegressionModel
+    from darts.models.forecasting.random_forest import RandomForest
+    from darts.models.forecasting.regression_model import RegressionModel
+    from darts.models.forecasting.regression_ensemble_model import RegressionEnsembleModel
+except ModuleNotFoundError:
+    logger.warning('Support for Regression based models (incl. RegressionEnsembleModel) not available, '
+                   'as they depend on PyTorch Datasets. '
+                   'To enable them, install "darts", "u8darts[torch]" or "u8darts[all]" (with pip); '
+                   'or "u8darts-torch" or "u8darts-all" (with conda).')
 
 try:
     from darts.models.forecasting.gradient_boosted_model import LightGBMModel
-except:
+except ModuleNotFoundError:
     logger.warning("Support for LightGBM not available. To enable LightGBM support in Darts, follow the detailed "
                    "install instructions for LightGBM in the README: "
                    "https://github.com/unit8co/darts/blob/master/README.md")
@@ -51,7 +62,6 @@ except:
 # Ensembling
 from darts.models.forecasting.ensemble_model import EnsembleModel
 from darts.models.forecasting.baselines import NaiveEnsembleModel
-from darts.models.forecasting.regression_ensemble_model import RegressionEnsembleModel
 
 # Filtering
 from darts.models.filtering.moving_average import MovingAverage
