@@ -11,15 +11,13 @@ The main functions are `fit()` and `predict()`. `fit()` learns the function `f()
 one or several time series. The function `predict()` applies `f()` on one or several time series in order
 to obtain forecasts for a desired number of time stamps into the future.
 """
-
-
+import copy
 from typing import Optional, Tuple, Union, Any, Callable, Dict, List, Sequence
 from itertools import product
-from abc import ABC, abstractmethod
-from inspect import signature
+from abc import ABC, ABCMeta, abstractmethod
+from random import sample
 import numpy as np
 import pandas as pd
-import xarray as xr
 
 from darts.timeseries import TimeSeries
 from darts.logging import get_logger, raise_log, raise_if_not, raise_if
@@ -29,6 +27,7 @@ from darts.utils import (
     _historical_forecasts_general_checks,
     _parallel_apply
 )
+from darts.utils.timeseries_generation import _generate_index
 import inspect
 
 from darts import metrics
@@ -36,14 +35,20 @@ from darts import metrics
 logger = get_logger(__name__)
 
 
-class ForecastingModel(ABC):
-    """ The base class for forecasting models. It defines the *minimal* behavior that all forecasting models have to support.
-        The signatures in this base class are for "local" models handling only one univariate series and no covariates.
-        Sub-classes can handle more complex cases.
+class ModelMeta(ABCMeta):
+    def __call__(cls, *args, **kwargs):
+        cls.model_call = (args, kwargs)
+        return super(ModelMeta, cls).__call__(*args, **kwargs)
+
+
+class ForecastingModel(ABC, metaclass=ModelMeta):
+    """ The base class for forecasting models. It defines the *minimal* behavior that all forecasting models have to
+        support. The signatures in this base class are for "local" models handling only one univariate series and no
+        covariates. Sub-classes can handle more complex cases.
     """
 
     @abstractmethod
-    def __init__(self):
+    def __init__(self, *args, **kwargs):
         # The series used for training the model through the `fit()` function.
         # This is only used if the model has been fit on one time series.
         self.training_series: Optional[TimeSeries] = None
@@ -51,6 +56,9 @@ class ForecastingModel(ABC):
         # state; whether the model has been fit (on a single time series)
         self._fit_called = False
 
+        # extract and store sub class model creation parameters
+        self._model_params = self._extract_model_creation_params()
+
     @abstractmethod
     def fit(self, series: TimeSeries) -> None:
         """ Trains the model on the provided univariate series.
@@ -88,6 +96,15 @@ class ForecastingModel(ABC):
         """
         return False
 
+    def _supports_non_retrainable_historical_forecasts(self) -> bool:
+        """
+        Checks if the forecasting model supports historical forecasts without retraining
+        the model. By default, returns False. Needs to be overwritten by models that do
+        support historical forecasts without retraining.
+        """
+        return False
+
+
     @property
     def uses_past_covariates(self):
         return 'past_covariates' in inspect.signature(self.fit).parameters.keys()
@@ -150,18 +167,9 @@ class ForecastingModel(ABC):
         """
         input_series = input_series if input_series is not None else self.training_series
 
-        if input_series.has_datetime_index:
-            # time_index = input_series.time_index
-            # new_dates = [
-            #     (time_index[-1] + (i * input_series.freq)) for i in range(1, n + 1)
-            # ]
-            # return pd.DatetimeIndex(new_dates, freq=input_series.freq_str)
-            start_time = input_series.end_time() + input_series.freq
-            return pd.date_range(start=start_time,
-                                 end=start_time + (n-1) * input_series.freq,
-                                 freq=input_series.freq)
-        else:
-            return pd.RangeIndex(start=input_series.end_time() + 1, stop=input_series.end_time() + n + 1, step=1)
+        last = input_series.end_time()
+        start = last + input_series.freq if input_series.has_datetime_index else last + 1
+        return _generate_index(start=start, freq=input_series.freq, length=n)
 
     def _build_forecast_series(self,
                                points_preds: Union[np.ndarray, Sequence[np.ndarray]],
@@ -291,6 +299,14 @@ class ForecastingModel(ABC):
         #     raise_if_not(series.end_time() <= covariates.end_time() and covariates.start_time() <= series.start_time(),
         #                  'The provided covariates must be at least as long as the target series.')
 
+        # only GlobalForecastingModels support historical forecastings without retraining the model
+        base_class_name = self.__class__.__base__.__name__
+        raise_if(not retrain and not self._supports_non_retrainable_historical_forecasts(),
+                 f'{base_class_name} does not support historical forecastings with `retrain` set to `False`. '
+                 f'For now, this is only supported with GlobalForecastingModels such as TorchForecastingModels. '
+                 f'Fore more information, read the documentation for `retrain` in `historical_forecastings()`',
+                 logger)
+
         # prepare the start parameter -> pd.Timestamp
         start = series.get_timestamp_at_point(start)
 
@@ -464,7 +480,8 @@ class ForecastingModel(ABC):
                    metric: Callable[[TimeSeries, TimeSeries], float] = metrics.mape,
                    reduction: Callable[[np.ndarray], float] = np.mean,
                    verbose=False,
-                   n_jobs: int = 1) -> Tuple['ForecastingModel', Dict]:
+                   n_jobs: int = 1,
+                   n_random_samples: Optional[Union[int, float]] = None) -> Tuple['ForecastingModel', Dict]:
         """
         A function for finding the best hyper-parameters among a given set.
         This function has 3 modes of operation: Expanding window mode, split mode and fitted value mode.
@@ -543,6 +560,14 @@ class ForecastingModel(ABC):
             The number of jobs to run in parallel. Parallel jobs are created only when there are two or more parameters
             combinations to evaluate. Each job will instantiate, train, and evaluate a different instance of the model.
             Defaults to `1` (sequential). Setting the parameter to `-1` means using all the available cores.
+        n_random_samples
+            The number/ratio of hyperparameter combinations to select from the full parameter grid. This will perform 
+            a random search instead of using the full grid. 
+            If an integer, `n_random_samples` is the number of parameter combinations selected from the full grid and must
+            be between `0` and the total number of parameter combinations.
+            If a float, `n_random_samples` is the ratio of parameter combinations selected from the full grid and must be
+            between `0` and `1`.
+            Defaults to `None` for which random selection will be ignored.
 
         Returns
         -------
@@ -569,9 +594,15 @@ class ForecastingModel(ABC):
         #     raise_if_not(series.has_same_time_as(covariates), 'The provided series and covariates must have the '
         #                                                       'same time axes.')
 
+
         # compute all hyperparameter combinations from selection
         params_cross_product = list(product(*parameters.values()))
 
+        #If n_random_samples has been set, randomly select a subset of the full parameter cross product to search with
+        if n_random_samples is not None:
+            params_cross_product = model_class._sample_params(params_cross_product, n_random_samples)
+            
+
         # iterate through all combinations of the provided parameters and choose the best one
         iterator = _build_tqdm_iterator(zip(params_cross_product), verbose, total=len(params_cross_product))
 
@@ -661,6 +692,31 @@ class ForecastingModel(ABC):
 
         return residuals
 
+    @classmethod
+    def _sample_params(model_class, params, n_random_samples):
+        """Select the absolute number of samples randomly if an integer has been supplied. If a float has been
+        supplied, select a fraction"""
+
+        if isinstance(n_random_samples, int):
+            raise_if_not((n_random_samples > 0) and (n_random_samples <= len(params)),
+                         "If supplied as an integer, n_random_samples must be greater than 0 and less than or equal to the size of the cartesian product of the hyperparameters.")
+            return sample(params, n_random_samples)
+
+        if isinstance(n_random_samples, float):
+            raise_if_not((n_random_samples > 0.0) and (n_random_samples <= 1.0),
+                         "If supplied as a float, n_random_samples must be greater than 0.0 and less than 1.0.")
+            return sample(params, int(n_random_samples * len(params)))
+
+    def _extract_model_creation_params(self):
+        """extracts immutable model creation parameters from `ModelMeta` and deletes reference."""
+        model_params = copy.deepcopy(self.model_call)
+        del self.__class__.model_call
+        return model_params
+
+    def untrained_model(self):
+        args, kwargs = self._model_params
+        return self.__class__(*args, **kwargs)
+
 
 class GlobalForecastingModel(ForecastingModel, ABC):
     """ The base class for "global" forecasting models, handling several time series and optional covariates.
@@ -800,6 +856,11 @@ class GlobalForecastingModel(ForecastingModel, ABC):
             future_covariates=future_covariates if self.uses_future_covariates else None
         )
 
+    def _supports_non_retrainable_historical_forecasts(self) -> bool:
+        """GlobalForecastingModel supports historical forecasts without retraining the model"""
+        return True
+
+
 class DualCovariatesForecastingModel(ForecastingModel, ABC):
     """ The base class for the forecasting models that are not global, but support future covariates.
     Among other things, it lets Darts forecasting models wrap around statsmodels models
