@@ -3,6 +3,7 @@ Additional util functions
 -------------------------
 """
 import pandas as pd
+import numpy as np
 
 from ..timeseries import TimeSeries
 from ..logging import raise_log, get_logger, raise_if_not, raise_if
@@ -186,7 +187,7 @@ def _historical_forecasts_general_checks(series, kwargs):
         elif isinstance(n.start, pd.Timestamp):
             raise_if(n.start not in series, '`start` timestamp must be an entry in the time series\' time index')
             raise_if(n.start == series.end_time(), '`start` timestamp is the last timestamp of the series', logger)
-        elif isinstance(n.start, int):
+        elif isinstance(n.start, (int, np.int64)):
             raise_if_not(n.start >= 0, logger)
             raise_if(n.start > len(series), '`start` index should be smaller than length of the series', logger)
         else:
